/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2004 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package org.openmobileis.embedded.webserver.templates;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import freemarker.template.Compileable;
import freemarker.template.TemplateException;
import freemarker.template.TextEncoding;
import freemarker.template.cache.CacheRetriever;
import freemarker.template.cache.Cacheable;
import freemarker.template.cache.RegistryAccepter;
import freemarker.template.cache.TemplateRegistry;

/**
 * Retrieves cacheable objects through the file system. This is the default
 * retriever for <code>FileTemplateCache</code>.
 *
 * @version $Id: FileRetriever.java,v 1.17 2004/10/24 09:14:42 run2000 Exp $
 */
public class Freemarkerv1JarCacheRetriever implements CacheRetriever,
        TextEncoding, RegistryAccepter {

    /** The root directory where the retriever will get files. */
    protected String directoryRoot;
    /** The filename suffix required for a file to be retrieved. */
    protected String filenameSuffix;
    /** The text encoding of the template files. */
    protected String encoding;
    /** The template registry to use to instantiate objects. */
    protected TemplateRegistry registry;
    
  	private ClassLoader loader;


    /** Creates new FileRetriever. */
    public Freemarkerv1JarCacheRetriever(ClassLoader loader) {
    	super();
  		this.loader = loader;
   }

    /**
     * Constructs a FileRetriever with a directory in which
     * it will look for template files.
     *
     * @param path the absolute path of the directory containing
     * templates for this retriever
     * @throws IllegalArgumentException the root directory is null
     */
    public Freemarkerv1JarCacheRetriever(String path, ClassLoader loader) {
    	this(loader);
      setConnection(path);
    }

    /**
     * Corresponds to checkCacheDir for file-system implementations.
     *
     * @return <code>true</code> if the connection is ok, otherwise <code>false</code>
     * @throws TemplateException the directory no longer exists, or is not
     * a directory
     */
    public boolean connectionOk() throws TemplateException {
        if( directoryRoot == null ) {
            throw new TemplateException( "Root directory is not defined" );
        }
        return true;
    }

    /**
     * Sets the root directory for this retriever.
     *
     * @param path the absolute path of the directory containing files
     * for this retriever.
     * @throws IllegalArgumentException the root directory is null
     */
    public void setConnection(String path) {
        if(path == null) {
            throw new IllegalArgumentException("Root cache path cannot be null");
        }
        directoryRoot = path;
    }

    /**
     * Gets the connection for this retriever. Corresponds to getPath for
     * file-system implementations.
     */
    public String getConnection() {
        if( directoryRoot == null ) {
            return null;
        }
        return directoryRoot.toString();
    }

    /**
     * Sets the root directory for this retriever.
     *
     * @param dir the root directory containing files for this retriever
     * @throws IllegalArgumentException the root directory is null
     */
    public void setPath(File dir) {
        if(dir == null) {
            throw new IllegalArgumentException("Root cache directory cannot be null");
        }
        this.directoryRoot = dir.getPath();
    }

    /**
     * Returns the root directory for this retriever.
     *
     * @return the root directory containing files for this retriever
     */
    public File getPath() {
        return new File(directoryRoot);
    }

    /**
     * Sets the file suffix. If set, files that do not have this suffix
     * will be ignored when read into the cache.
     *
     * @param filenameSuffix the optional filename suffix of
     * files to be read for this retriever.
     */
    public void setFilenameSuffix(String filenameSuffix) {
        this.filenameSuffix = filenameSuffix;
    }

    /**
     * Returns the file suffix. If set, files that do not have this suffix
     * will be ignored when read into the cache.
     *
     * @return the optional filename suffix of files to be read
     * for this retriever.
     */
    public String getFilenameSuffix() {
        return filenameSuffix;
    }

    /**
     * Tests whether the object still exists in the template repository.
     * This may be redundant. Instead, lastModified could throw an appropriate
     * exception.
     *
     * @param location the location of the object to be tested
     * @return <code>true</code> if the object still exists in the repository,
     * otherwise <code>false</code>
     * @see #lastModified
     */
    public boolean exists(String location) {

        try {
        	InputStream in = this.getClass().getClassLoader().getResourceAsStream(nameToFile(location));
          return(in != null);
        } catch( Throwable e ) {
            return false;
        }
    }

    /**
     * Returns a list of objects (<code>String</code>s) to pre-load the cache with.
     *
     * @return a <code>List</code> of <code>String</code>s to preload the
     * cache with
     */
    public List getPreloadData() throws TemplateException {
        List visitedFiles = new LinkedList();

        try {
            readDirectory(directoryRoot, "", visitedFiles);
        } catch (IOException e) {
            throw new TemplateException( "Could not get preload data", e );
        }
        return visitedFiles;
    }

    /**
     * Recursively updates the cache from the files in a (sub)directory
     * and its subdirectories.
     *
     * @param dir the directory to be read.
     * @param relativeDirPath a string representing the directory's path
     * relative to the root cache directory.
     * @param visitedFiles a List of files that have been visited so far.
     */
    protected void readDirectory(String dir, String relativeDirPath,
            List visitedFiles) throws IOException {
    }

    /**
     * <p>Determines when the object in the template repository was last modified.</p>
     *
     * @throws TemplateException is thrown whenever the item:
     * <ul>
     * <li> does not exist </li>
     * <li> is the wrong type (eg. directory, not file) </li>
     * <li> has an invalid file suffix </li>
     * </ul>
     */
    public long lastModified(String location) throws TemplateException {
         return 0;
    }

    /**
     * Determine whether the filename ends with the appropriate filename
     * suffix.
     * @param name the filename to be checked
     * @return is the filename suffix ok?
     * @throws TemplateException the suffix is invalid
     */
    protected boolean isSuffixValid( String name ) throws TemplateException {
        if (!(filenameSuffix == null || name.endsWith(filenameSuffix))) {
            throw new TemplateException("The requested name, \"" + name +
                    "\", does not have the filename suffix \"" +
                    filenameSuffix + '"');
        }
        return true;
    }

    /**
     * Converts a cache element name to a <tt>File</tt>.
     * @param name the filename relative to the directory root of the retriever
     * @return the fully qualified filename
     */
    protected String nameToFile(final String name) throws TemplateException {
        return directoryRoot+name;
    }

    /**
     * Retrieves the appropriate data to be stored in the cache.
     *
     * @param location the filename, relative to the root directory, of the
     * template data to load
     * @param type the type of item to be loaded
     * @return the template data
     */
    public Cacheable loadData(String location, String type)
            throws TemplateException {
        String    file = nameToFile(location);

        try {
          InputStream inputStream = this.loader.getResourceAsStream(file);
          Compileable template = (Compileable)registry.getTemplate( type );

            if (encoding == null) {
                template.compileFromStream(inputStream);
            } else {
                template.compileFromStream(inputStream, encoding);
            }
            inputStream.close();
            return (Cacheable)template;
        } catch(java.io.IOException e) {
            throw new TemplateException( "Could not load data", e );
        } catch(NullPointerException e) {
            throw new TemplateException( "Could not load data", e );
        }
    }

    /**
     * Sets the character encoding to be used when reading template files.
     *
     * @param encoding the name of the encoding to be used; this will be
     * passed to the constructor of <tt>InputStreamReader</tt>.
     */
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    /**
     * Returns the character encoding to be used when reading template files.
     *
     * @return the name of the encoding to be used; this will be
     * passed to the constructor of <tt>InputStreamReader</tt>.
     */
    public String getEncoding() {
        return encoding;
    }

    /**
     * Sets a template registry implementation to use when creating new
     * templates.
     */
     public void setTemplateRegistry( TemplateRegistry cRegistry ) {
         registry = cRegistry;
     }

    /**
     * Gets the current template registry implementation in use.
     */
     public TemplateRegistry getTemplateRegistry() {
         return registry;
     }

    /**
     * Is this file retriever equal to another object?
     *
     * @param o the object to compare this object with
     * @return <code>true</code> if the objects are equal, otherwise
     * <code>false</code>
     */
    public boolean equals( Object o ) {
        if( this == o )
            return true;

        if( !( o instanceof Freemarkerv1JarCacheRetriever ))
            return false;

        final Freemarkerv1JarCacheRetriever fileRetriever = ( Freemarkerv1JarCacheRetriever ) o;

        if( directoryRoot == null ?
                fileRetriever.directoryRoot != null :
                !directoryRoot.equals( fileRetriever.directoryRoot ))
            return false;
        if( encoding == null ?
                fileRetriever.encoding != null :
                !encoding.equals( fileRetriever.encoding ))
            return false;
        if( filenameSuffix == null ?
                fileRetriever.filenameSuffix != null :
                !filenameSuffix.equals( fileRetriever.filenameSuffix ))
            return false;
        if( registry == null ?
                fileRetriever.registry != null :
                !registry.equals( fileRetriever.registry ))
            return false;

        return true;
    }

    /**
     * Retrieve the hash code for this object
     * @return the hash code
     */
    public int hashCode() {
        int result = 13;
        result = 29 * result + ( directoryRoot != null ? directoryRoot.hashCode() : 0 );
        result = 29 * result + ( filenameSuffix != null ? filenameSuffix.hashCode() : 0 );
        result = 29 * result + ( encoding != null ? encoding.hashCode() : 0 );
        result = 29 * result + ( registry != null ? registry.hashCode() : 0 );
        return result;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a <code>String</code> representation of the object
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();

        if( directoryRoot != null ) {
            buffer.append( "Root path: " );
            buffer.append( directoryRoot );
        }
        if( filenameSuffix != null ) {
            buffer.append( ", filename suffix: " );
            buffer.append( filenameSuffix );
        }
        if( encoding != null ) {
            buffer.append( ", encoding: " );
            buffer.append( encoding );
        }
        if( registry != null ) {
            buffer.append( ", registry: " );
            buffer.append( registry );
        }
        return buffer.toString();
    }
}
