/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  2004 Creation P.Delrieu
 *
 */

package org.openmobileis.services.common;

import java.io.IOException;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.modules.profiles.terminal.RubricLoader;
import org.openmobileis.services.Service;
import org.openmobileis.services.servlet.OpenmisHttpServletRequest;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Load all known services at launching. Names and
 * classes are described in a property file and set at the initialization (init() method) by
 * the web server.
 * Each time an objecthas to access to a service, call getService from Service manager.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class ServiceManager implements ServiceManagerService {

  private static ServiceManager manager;
  // key : service name, value : service instance
  private java.util.Hashtable services = new java.util.Hashtable(10);
  private String baseURI;// = "/services";


  protected ServiceManager() {
  }

  public static ServiceManager getManager() {
  	if (manager == null)	{
  		synchronized(ServiceManager.class)	{
				if (manager == null)	{
					manager = new ServiceManager();
				}
  		}
  	}
    return manager;
  }

  public static void registerManager(ServiceManager servicemanager)	{
  	if (manager == null) {
  		manager = servicemanager;
			ApplicationContextManager.getManager().addManager(manager);
  	} else	{
  		LogManager.traceDebug(0, "ServiceManager registerManager already done no use");
  	}
  }

  // init called by the web server at the initialization phase (new Serve())
  public void init (java.util.Properties props, String baseURI) throws ServiceException {
       // Init baseURI
        this.baseURI = baseURI;

		// load services

		 ReturnLinkManagerService returnlinkservice = new ReturnLinkManagerService();
		 String returnlinkname = returnlinkservice.getServiceUri().substring(this.baseURI.length(), returnlinkservice.getServiceUri().length());
		 services.put(returnlinkname, returnlinkservice);

		 if (props != null)	{
			java.util.Enumeration enume = props.propertyNames();
			while (enume.hasMoreElements()) {
				 String serviceFileName = (String)enume.nextElement();
				 String className = props.getProperty(serviceFileName);
					 // Check if we're allowed to make one of these.
				 try {
						 Object toload = Class.forName( className ).newInstance();
						 if (toload instanceof Service) {
							 Service service = (Service)toload;
	             this.loadService(service);
						 } else if (toload instanceof RubricLoader){
	             RubricLoader module = (RubricLoader)toload;
							 this.loadRubricLoader(module);
						 } else {
						     LogManager.traceError(LogServices.WEBSERVICE, "Try to load a class which is neither a service, neither a module loader : "+className);
						 }
					 } catch ( ClassNotFoundException e )  {
							LogManager.traceError(LogServices.WEBSERVICE, "Class not found: " + className );
							throw new ServiceException(e);
					 }	catch ( ClassCastException e )  {
						 LogManager.traceError(LogServices.WEBSERVICE, "Class cast problem: " + e.getMessage());
	           throw new ServiceException(e);
					 }	catch ( InstantiationException e ) {
						 LogManager.traceError(LogServices.WEBSERVICE, "Instantiation problem: " + e.getMessage() );
	           throw new ServiceException(e);
					 }	catch ( IllegalAccessException e )  {
						 LogManager.traceError(LogServices.WEBSERVICE, "Illegal class access: " + e.getMessage());
	           throw new ServiceException(e);
					 }	catch ( Exception e )  {
						 LogManager.traceError(LogServices.WEBSERVICE, "Unexpected problem creating servlet: " + e );
	           throw new ServiceException(e);
					} // end catch
				} //end while
		 }
  }

  public void loadService(Service service) {
    String serviceURI = service.getServiceUri();
    if (serviceURI.startsWith(ServiceManager.getManager().getServiceBaseURI())) {
      serviceURI = serviceURI.substring(ServiceManager.getManager().getServiceBaseURI().length(), serviceURI.length());
    }
     services.put(serviceURI, service );
  }

  public void unloadService(String serviceURI)	{
    if (serviceURI.startsWith(ServiceManager.getManager().getServiceBaseURI())) {
      serviceURI = serviceURI.substring(ServiceManager.getManager().getServiceBaseURI().length(), serviceURI.length());
    }
  	Object ok = services.remove(serviceURI);
  	if (ok == null) LogManager.traceDebug(0, "erreur service non unregistrer "+serviceURI);
  }

  public Array loadRubricLoader(RubricLoader rubricLoader) throws ServiceException  {
    rubricLoader.preLoadingInit();

    // load services
    Array serviceList = rubricLoader.loadService();
    for (int i=0; i<serviceList.size(); i++)  {
      Service service = (Service)serviceList.get(i);
      this.loadService(service);
    }

    // load listener
    rubricLoader.postLoadingInit();
    return serviceList;
  }

  public String getServiceBaseURI()	{
  	return this.baseURI;
  }

	public void setServiceBaseURI(String uri)	{
		this.baseURI = uri;
	}


  /*
  * Return the Service called serviceName. Return null if it is not registered
  * @param : String service name
  * @return: Service or null
  */
  private Service getService (String serviceName) {
    // try to get the service from URL of type /serviceName/otherPath/....
    Service service =  (Service)services.get(serviceName);
    if (service == null)    {
        int index  = serviceName.lastIndexOf('/');
        while (index != -1) {
           serviceName = serviceName.substring(0, index);
           service =  (Service)services.get(serviceName);
           if (service != null) {
            break;
           }
           index  = serviceName.lastIndexOf('/');
        }
    }
    return service;
  }

  /*
   * Return the Service with its URI. Return null if it is not registered
   * @param : String service URI
   * @return: Service if found or null
   */
  public Service getServiceByURI(String serviceURI) {
	  if (serviceURI.startsWith(baseURI)) {
      String serviceName = serviceURI.substring(ServiceManager.getManager().getServiceBaseURI().length(), serviceURI.length());
      int index = serviceName.indexOf("?");
      if (index != -1)	{
      	serviceName = serviceName.substring(0, index);
      }
		  return this.getService(serviceName);
	  } else {
		  return this.getService(serviceURI);
	  }
  }

  public void redirectToServiceURI(String uri, HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
	  Service service = this.getServiceByURI(uri);
    if (!uri.startsWith(baseURI)) {
      uri=baseURI+uri;
    }
    //parse URL to get parameters
    int index = uri.indexOf("?");
    if (index != -1){
    	String parameterstr = uri.substring(index+1, uri.length());
    	index = parameterstr.indexOf("#");
    	if (index != -1)	{ //remove remainding anchor #
    		parameterstr = parameterstr.substring(0, index);
    	}
    	StringTokenizer token = new StringTokenizer(parameterstr, "&");
    	Map parammap = req.getParameterMap();
    	while (token.hasMoreTokens())	{
    		String paramval = token.nextToken();
    		index = paramval.indexOf("=");
    		if (index != -1)	{
    			String value = paramval.substring(0, index);
    			String param[] = new String[]{paramval.substring(index+1, paramval.length())};
    			parammap.put(value, param);
    		}
    	}
    }

    if (req instanceof OpenmisHttpServletRequest)	{
	    OpenmisHttpServletRequest request = (OpenmisHttpServletRequest)req;
	    request.setRequestURI(uri);
    }
	  service.runService(req, res);
  }

  public CallingServiceManager getCallingServiceManager()	{
		CallingServiceManager callServManager = CallingServiceManager.getManager();
		return callServManager;
  }

}